package synth

import (
	"fmt"
	"log"
	"os/exec"
)

func CodeCheck(fileName string, fileFolder string) (string, error) {
	out, err := executeIverilog(fileName, fileFolder)
	if err != nil {
		fmt.Printf("%s \n", out)
		return out, err
	}

	err = executeYosys(fileName, fileFolder)
	if err != nil {
		fmt.Printf("%s \n", err)
		return "Somthing went wrong: ", err
		// executeFSM(fileName)
	}
	// executeNetlistSVG(fileName)
	return "", nil

}

func RunSynth(fileName string, fileFolder string) {
	// err := CodeCheck(fileName)
	// if err != nil {
	// 	return
	// 	// executeFSM(fileName)
	// }
	synth(fileName, fileFolder)
	placeAndRouter(fileName)
	binGenerate(fileName)

}

func executeYosys(fileName string, fileFolder string) error {
	app := "yosys"
	readFile := "-p read_verilog " + "../files/" + fileFolder + "/" + fileName + ".v"
	hierarchy := "-p hierarchy -check"
	// highlevels := "-p proc; opt; fsm; opt; memory; opt"
	highlevel := "-p proc; opt; fsm -nomap; opt; memory; opt"
	// prep := "-p prep -flatten"
	prep := "-p prep -auto-top -ifx -flatten"
	fsm := "-p fsm_export -o ../fsm/" + fileName + ".kiss2"
	// techmap := "-p techmap"
	// extract := "-p extract -map ../verilog/coarse_cells.v"
	outputJson := "-p write_json " + "../json/" + fileName + ".json "
	// show := "-p show"
	arg0 := []string{readFile, hierarchy, highlevel, prep, outputJson, fsm}
	yosysCmd := exec.Command(app, arg0...)
	err := yosysCmd.Run()
	if err != nil {
		log.Printf("yosysCmd.Run() failed with '%s'\n", err)
		return err
	}
	fmt.Println("Ferdig med Yosys")
	return nil

}

func executeIverilog(fileName string, fileFolder string) (string, error) {
	app := "iverilog"
	fmt.Println(fileFolder)
	arg0 := []string{"../files/" + fileFolder + "/" + fileName + ".v"}
	iverilogCmd := exec.Command(app, arg0...) //, arg1, arg2)
	out, err := iverilogCmd.CombinedOutput()
	if err != nil {

		log.Printf("iverilog.Run() failed with '%s'\n", err)
		return string(out), err
	}
	if len(out) != 0 {
		fmt.Printf("%s \n", out)
	}
	fmt.Print("Ferdig med Iverilog \n")
	return "", nil
}

func NetlistSVG(fileName string) {
	app := "netlistsvg"

	arg0 := []string{"../json/" + fileName + ".json", "-o", "../svg/" + fileName + ".svg"}

	netlistsvgCmd := exec.Command(app, arg0...)
	out, err := netlistsvgCmd.CombinedOutput()
	if err != nil {
		log.Printf("netlistsvgCmd.Run() failed with '%s'\n", err)
	}

	if len(out) != 0 {
		fmt.Printf("%s \n", out)
	}
	fmt.Print("Ferdig med netlistsvg\n")
}

func executeFSM(fileName string) {
	app := "python3"

	// arg0 := []string{"../fsm/kiss2dot.py", "../fsm/" + fileName + ".kiss2", ">", "../dot/" + fileName + ".dot"}
	arg0 := []string{"../fsm/kiss2dot.py", "../fsm/fsm.kiss2", ">", "../dot/fsm.dot"}

	// arg0 := []string{"../fsm/kiss2dot.py", "../fsm/fsm.kiss2"}
	fmt.Println(arg0)
	netlistsvgCmd := exec.Command(app, arg0...)

	out, err := netlistsvgCmd.CombinedOutput()
	if err != nil {
		log.Printf("executeFSM.Run() failed with '%s'\n", err)
	}

	if len(out) != 0 {
		fmt.Printf("%s \n", out)
	}
	fmt.Print("Ferdig med fsm\n")
}

func synth(fileName string, fileFolder string) {
	app := "yosys"
	fmt.Printf("\n Dette er folder: %s\n", fileFolder)
	arg0 := []string{"-p", "synth_ice40 -blif ../blif/" + fileName + ".blif", "../files/" + fileFolder + "/" + fileName + ".v"}
	fmt.Printf("\n Dette er arg: %s\n", arg0)
	synthCmd := exec.Command(app, arg0...)
	// out, err := synthCmd.CombinedOutput()
	err := synthCmd.Run()
	if err != nil {
		log.Printf("synthCmd.Run() failed with '%s'\n", err)
	}
	// fmt.Println(string(out))
}

func placeAndRouter(fileName string) {
	app := "arachne-pnr"
	// arg0 := []string{"-d", "1k", "-P", "vq100", "-o", "../asc/" + fileName + ".asc", "-p", "../pcf/iceblink.pcf", "../blif/" + fileName + ".blif"}
	arg0 := []string{"-d", "1k", "-P", "tq144", "-o", "../asc/" + fileName + ".asc", "-p", "../pcf/icestick.pcf", "../blif/" + fileName + ".blif"}
	pARCmd := exec.Command(app, arg0...)
	// out, err := pARCmd.CombinedOutput()
	err := pARCmd.Run()
	if err != nil {
		log.Printf("pARCmd.Run() failed with '%s'\n", err)
	}
	// fmt.Println(string(out))
}

func binGenerate(fileName string) {
	app := "icepack"
	arg0 := []string{"../asc/" + fileName + ".asc", "../bin/" + fileName + ".bin"}
	binCmd := exec.Command(app, arg0...)
	err := binCmd.Run()
	if err != nil {
		log.Printf("binCmd.Run() failed with '%s'\n", err)
	}
}

func ProgDevice(fileName string) {
	app := "iceprog"
	arg0 := []string{"../bin/" + fileName + ".bin"}
	progCmd := exec.Command(app, arg0...)
	out, err := progCmd.CombinedOutput()
	if err != nil {
		log.Printf("progCmd.Run() failed with '%s'\n", err)
	}
	fmt.Println(string(out))
}
